@@ -4,9 +4,10 @@
 @Contact :   g1879@qq.com
 """
 from re import search
-from time import sleep
+from time import sleep, perf_counter
+from warnings import warn
 
-from .chromium_base import ChromiumBase
+from .chromium_base import ChromiumBase, ChromiumPageScroll, ChromiumBaseSetter
 from .chromium_element import ChromiumElement
 
 
@@ -14,10 +15,11 @@ class ChromiumFrame(ChromiumBase):
     def __init__(self, page, ele):
         self.page = page
         self.address = page.address
-        node = page.run_cdp('DOM.describeNode', nodeId=ele.node_id, not_change=True)['node']
+        node = page.run_cdp('DOM.describeNode', backendNodeId=ele.ids.backend_id)['node']
         self.frame_id = node['frameId']
-        self._backend_id = ele.backend_id
+        self._backend_id = ele.ids.backend_id
         self._frame_ele = ele
+        self._states = None
 
         if self._is_inner_frame():
             self._is_diff_domain = False
@@ -28,6 +30,11 @@ class ChromiumFrame(ChromiumBase):
             super().__init__(page.address, self.frame_id, page.timeout)
             obj_id = super().run_js('document;', as_expr=True)['objectId']
             self.doc_ele = ChromiumElement(self, obj_id=obj_id)
+        self._ids = ChromiumFrameIds(self)
+
+        end_time = perf_counter() + 2
+        while perf_counter() < end_time and self.url == 'about:blank':
+            sleep(.1)
 
     def __call__(self, loc_or_str, timeout=None):
         """在内部查找元素
@@ -43,46 +50,54 @@ class ChromiumFrame(ChromiumBase):
         attrs = [f"{attr}='{attrs[attr]}'" for attr in attrs]
         return f'<ChromiumFrame {self.frame_ele.tag} {" ".join(attrs)}>'
 
-    def _set_options(self):
+    def _runtime_settings(self):
         """重写设置浏览器运行参数方法"""
         self._timeouts = self.page.timeouts
         self._page_load_strategy = self.page.page_load_strategy
 
-    def _init_page(self, tab_id=None):
+    def _driver_init(self, tab_id):
         """避免出现服务器500错误
         :param tab_id: 要跳转到的标签页id
         :return: None
         """
         try:
-            super()._init_page(tab_id)
+            super()._driver_init(tab_id)
         except:
             self._control_session.get(f'http://{self.address}/json')
-            super()._init_page(tab_id)
+            super()._driver_init(tab_id)
 
     def _reload(self):
+        """重新获取document"""
+        debug = self._debug
+        if debug:
+            print('reload')
+
         self._frame_ele = ChromiumElement(self.page, backend_id=self._backend_id)
-        node = self.page.run_cdp('DOM.describeNode', nodeId=self._frame_ele.node_id, not_change=True)['node']
+        node = self.page.run_cdp('DOM.describeNode', backendNodeId=self._frame_ele.ids.backend_id)['node']
 
         if self._is_inner_frame():
             self._is_diff_domain = False
             self.doc_ele = ChromiumElement(self.page, backend_id=node['contentDocument']['backendNodeId'])
             super().__init__(self.address, self.page.tab_id, self.page.timeout)
+            self._debug = debug
         else:
             self._is_diff_domain = True
             self._tab_obj.stop()
             super().__init__(self.address, self.frame_id, self.page.timeout)
             obj_id = super().run_js('document;', as_expr=True)['objectId']
             self.doc_ele = ChromiumElement(self, obj_id=obj_id)
+            self._debug = debug
 
     def _check_ok(self):
+        """用于应付同域异域之间跳转导致元素丢失问题"""
         if self._tab_obj._stopped.is_set():
             self._reload()
 
         try:
-            self._tab_obj.DOM.describeNode(nodeId=self.node_id)
+            self.page.run_cdp('DOM.describeNode', nodeId=self.ids.node_id)
         except Exception:
             self._reload()
-            sleep(.5)
+            # sleep(2)
 
     def _get_new_document(self):
         """刷新cdp使用的document数据"""
@@ -95,12 +110,11 @@ class ChromiumFrame(ChromiumBase):
             while True:
                 try:
                     if self._is_diff_domain is False:
-                        node = self.page.run_cdp('DOM.describeNode',
-                                                 backendNodeId=self.backend_id, not_change=True)['node']
+                        node = self.page.run_cdp('DOM.describeNode', backendNodeId=self.ids.backend_id)['node']
                         self.doc_ele = ChromiumElement(self.page, backend_id=node['contentDocument']['backendNodeId'])
 
                     else:
-                        b_id = self._tab_obj.DOM.getDocument()['root']['backendNodeId']
+                        b_id = self.run_cdp('DOM.getDocument')['root']['backendNodeId']
                         self.doc_ele = ChromiumElement(self, backend_id=b_id)
 
                     break
@@ -115,6 +129,14 @@ class ChromiumFrame(ChromiumBase):
             self._is_loading = False
             self._is_reading = False
 
+    def _onLoadEventFired(self, **kwargs):
+        """在页面刷新、变化后重新读取页面内容"""
+        # 用于覆盖父类方法，不能删
+        if self._debug:
+            print('loadEventFired')
+            if self._debug_recorder:
+                self._debug_recorder.add_data((perf_counter(), '加载流程', 'loadEventFired'))
+
     def _onFrameStartedLoading(self, **kwargs):
         """页面开始加载时触发"""
         if kwargs['frameId'] == self.frame_id:
@@ -130,24 +152,8 @@ class ChromiumFrame(ChromiumBase):
             self._get_new_document()
 
     @property
-    def tab_id(self):
-        """返回当前标签页id"""
-        return self.page.tab_id
-
-    @property
-    def backend_id(self):
-        """返回cdp中的node id"""
-        return self._backend_id
-
-    @property
-    def obj_id(self):
-        """返回frame元素的object id"""
-        return self.frame_ele.obj_id
-
-    @property
-    def node_id(self):
-        """返回cdp中的node id"""
-        return self.frame_ele.node_id
+    def ids(self):
+        return self._ids
 
     @property
     def frame_ele(self):
@@ -171,8 +177,7 @@ class ChromiumFrame(ChromiumBase):
         """返回元素outerHTML文本"""
         self._check_ok()
         tag = self.tag
-        out_html = self.page.run_cdp('DOM.getOuterHTML',
-                                     nodeId=self.frame_ele.node_id, not_change=True)['outerHTML']
+        out_html = self.page.run_cdp('DOM.getOuterHTML', backendNodeId=self.frame_ele.ids.backend_id)['outerHTML']
         sign = search(rf'<{tag}.*?>', out_html).group(0)
         return f'{sign}{self.inner_html}</{tag}>'
 
@@ -186,7 +191,8 @@ class ChromiumFrame(ChromiumBase):
     def title(self):
         """返回页面title"""
         self._check_ok()
-        return self.ele('t:title').text
+        r = self._ele('t:title', raise_err=False)
+        return r.text if r else None
 
     @property
     def cookies(self):
@@ -227,10 +233,9 @@ class ChromiumFrame(ChromiumBase):
         return self.frame_ele.location
 
     @property
-    def is_displayed(self):
-        """返回frame元素是否显示"""
-        self._check_ok()
-        return self.frame_ele.is_displayed
+    def locations(self):
+        """返回用于获取元素位置的对象"""
+        return self.frame_ele.locations
 
     @property
     def xpath(self):
@@ -260,6 +265,25 @@ class ChromiumFrame(ChromiumBase):
                 except:
                     pass
 
+    @property
+    def scroll(self):
+        """返回用于等待的对象"""
+        return ChromiumFrameScroll(self)
+
+    @property
+    def set(self):
+        """返回用于等待的对象"""
+        if self._set is None:
+            self._set = ChromiumFrameSetter(self)
+        return self._set
+
+    @property
+    def states(self):
+        """返回用于获取状态信息的对象"""
+        if self._states is None:
+            self._states = ChromiumFrameStates(self)
+        return self._states
+
     def refresh(self):
         """刷新frame页面"""
         self._check_ok()
@@ -273,15 +297,6 @@ class ChromiumFrame(ChromiumBase):
         self._check_ok()
         return self.frame_ele.attr(attr)
 
-    def set_attr(self, attr, value):
-        """设置frame元素attribute属性
-        :param attr: 属性名
-        :param value: 属性值
-        :return: None
-        """
-        self._check_ok()
-        self.frame_ele.set_attr(attr, value)
-
     def remove_attr(self, attr):
         """删除frame元素attribute属性
         :param attr: 属性名
@@ -290,15 +305,18 @@ class ChromiumFrame(ChromiumBase):
         self._check_ok()
         self.frame_ele.remove_attr(attr)
 
-    def run_js(self, script, as_expr=False, *args):
+    def run_js(self, script, *args, as_expr=False):
         """运行javascript代码
         :param script: js文本
-        :param as_expr: 是否作为表达式运行，为True时args无效
         :param args: 参数，按顺序在js文本中对应argument[0]、argument[1]...
+        :param as_expr: 是否作为表达式运行，为True时args无效
         :return: 运行的结果
         """
         self._check_ok()
-        return self.doc_ele.run_js(script, as_expr=as_expr, *args)
+        if script.startswith('this.scrollIntoView'):
+            return self.frame_ele.run_js(script, *args, as_expr=as_expr)
+        else:
+            return self.doc_ele.run_js(script, *args, as_expr=as_expr)
 
     def parent(self, level_or_loc=1):
         """返回上面某一级父元素，可指定层数或用查询语法定位
@@ -384,19 +402,39 @@ class ChromiumFrame(ChromiumBase):
         self._check_ok()
         return self.frame_ele.afters(filter_loc, timeout)
 
-    def _ele(self, loc_or_ele, timeout=None, single=True, relative=False):
+    def get_screenshot(self, path=None, as_bytes=None, full_page=False, left_top=None, right_bottom=None):
+        """对页面进行截图，可对整个网页、可见网页、指定范围截图。对可视范围外截图需要90以上版本浏览器支持
+        :param path: 完整路径，后缀可选 'jpg','jpeg','png','webp'
+        :param as_bytes: 是否已字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
+        :param full_page: 是否整页截图，为True截取整个网页，为False截取可视窗口
+        :param left_top: 截取范围左上角坐标
+        :param right_bottom: 截取范围右下角角坐标
+        :return: 图片完整路径或字节文本
+        """
+        if full_page:
+            raise RuntimeError('暂未实现对iframe全页截图功能。')
+        if left_top is None and right_bottom is None:
+            return self.frame_ele.get_screenshot(path=path, as_bytes=as_bytes)
+        else:
+            raise RuntimeError('暂未实现对异域iframe内元素截图功能。')
+
+    def _find_elements(self, loc_or_ele, timeout=None, single=True, relative=False, raise_err=None):
         """在frame内查找单个元素
         :param loc_or_ele: 定位符或元素对象
         :param timeout: 查找超时时间
+        :param single: True则返回第一个，False则返回全部
+        :param relative: WebPage用的表示是否相对定位的参数
+        :param raise_err: 找不到元素是是否抛出异常，为None时根据全局设置
         :return: ChromiumElement对象
         """
+        self._check_ok()
         if isinstance(loc_or_ele, ChromiumElement):
             return loc_or_ele
 
-        while self.is_loading:
-            sleep(.05)
+        self.wait.load_complete()
 
-        return self.doc_ele.ele(loc_or_ele, timeout) if single else self.doc_ele.eles(loc_or_ele, timeout)
+        return self.doc_ele._ele(loc_or_ele, timeout, raise_err=raise_err) \
+            if single else self.doc_ele.eles(loc_or_ele, timeout)
 
     def _d_connect(self, to_url, times=0, interval=1, show_errmsg=False, timeout=None):
         """尝试连接，重试若干次
@@ -413,11 +451,11 @@ class ChromiumFrame(ChromiumBase):
 
         for t in range(times + 1):
             err = None
-            result = self._driver.Page.navigate(url=to_url, frameId=self.frame_id)
+            result = self.driver.Page.navigate(url=to_url, frameId=self.frame_id)
 
             is_timeout = not self._wait_loaded(timeout)
-            while self.is_loading:
-                sleep(.1)
+            sleep(.5)
+            self.wait.load_complete()
 
             if is_timeout:
                 err = TimeoutError('页面连接超时。')
@@ -445,4 +483,107 @@ class ChromiumFrame(ChromiumBase):
 
     def _is_inner_frame(self):
         """返回当前frame是否同域"""
-        return self.frame_id in str(self.page.run_cdp('Page.getFrameTree', not_change=True)['frameTree'])
+        return self.frame_id in str(self.page.run_cdp('Page.getFrameTree')['frameTree'])
+
+    # -------------准备废弃------------
+    def set_attr(self, attr, value):
+        """设置frame元素attribute属性
+        :param attr: 属性名
+        :param value: 属性值
+        :return: None
+        """
+        warn("set_attr()方法即将弃用，请用set.attr()方法代替。", DeprecationWarning)
+        self.set.attr(attr, value)
+
+    @property
+    def is_displayed(self):
+        """返回frame元素是否显示"""
+        warn("is_displayed属性即将弃用，请用states.is_displayed属性代替。", DeprecationWarning)
+        return self.states.is_displayed()
+
+    @property
+    def tab_id(self):
+        """返回当前标签页id"""
+        warn("tab_id属性即将弃用，请用ids.tab_id属性代替。", DeprecationWarning)
+        return self.ids.tab_id
+
+    @property
+    def backend_id(self):
+        """返回cdp中的node id"""
+        warn("backend_id属性即将弃用，请用ids.backend_id属性代替。", DeprecationWarning)
+        return self.ids.backend_id
+
+    @property
+    def obj_id(self):
+        """返回frame元素的object id"""
+        warn("obj_id属性即将弃用，请用ids.obj_id属性代替。", DeprecationWarning)
+        return self.ids.obj_id
+
+    @property
+    def node_id(self):
+        """返回cdp中的node id"""
+        warn("node_id属性即将弃用，请用ids.node_id属性代替。", DeprecationWarning)
+        return self.ids.node_id
+
+
+class ChromiumFrameIds(object):
+    def __init__(self, frame):
+        self._frame = frame
+
+    @property
+    def tab_id(self):
+        """返回当前标签页id"""
+        return self._frame.page.tab_id
+
+    @property
+    def backend_id(self):
+        """返回cdp中的node id"""
+        return self._frame._backend_id
+
+    @property
+    def obj_id(self):
+        """返回frame元素的object id"""
+        return self._frame.frame_ele.ids.obj_id
+
+    @property
+    def node_id(self):
+        """返回cdp中的node id"""
+        return self._frame.frame_ele.ids.node_id
+
+
+class ChromiumFrameStates(object):
+    def __init__(self, frame):
+        self._frame = frame
+
+    def is_displayed(self):
+        """返回frame元素是否显示"""
+        self._frame._check_ok()
+        return self._frame.frame_ele.states.is_displayed
+
+
+class ChromiumFrameScroll(ChromiumPageScroll):
+    def __init__(self, frame):
+        """
+        :param frame: ChromiumFrame对象
+        """
+        self._driver = frame.doc_ele
+        self.t1 = self.t2 = 'this.documentElement'
+
+    def to_see(self, loc_or_ele):
+        """滚动页面直到元素可见
+        :param loc_or_ele: 元素的定位信息，可以是loc元组，或查询字符串
+        :return: None
+        """
+        ele = loc_or_ele if isinstance(loc_or_ele, ChromiumElement) else self._driver._ele(loc_or_ele)
+        ele.run_js('this.scrollIntoView({behavior: "auto", block: "center", inline: "center"});')
+
+
+class ChromiumFrameSetter(ChromiumBaseSetter):
+    def attr(self, attr, value):
+        """设置frame元素attribute属性
+        :param attr: 属性名
+        :param value: 属性值
+        :return: None
+        """
+        self._page._check_ok()
+        self._page.frame_ele.set.attr(attr, value)
