@@ -6,6 +6,7 @@
 from re import search
 from time import sleep
 from urllib.parse import urlparse
+from warnings import warn
 
 from DownloadKit import DownloadKit
 from requests import Session, Response
@@ -13,8 +14,8 @@ from requests.structures import CaseInsensitiveDict
 from tldextract import extract
 
 from .base import BasePage
+from .commons.web import cookie_to_dict, set_session_cookies
 from .configs.session_options import SessionOptions
-from .functions.web import cookie_to_dict, set_session_cookies
 from .session_element import SessionElement, make_session_ele
 
 
@@ -28,71 +29,36 @@ class SessionPage(BasePage):
         """
         self._response = None
         self._download_set = None
-        self._create_session(session_or_options)
+        self._session = None
+        self._set = None
+        self._set_start_options(session_or_options, None)
+        self._set_runtime_settings()
+        self._create_session()
         timeout = timeout if timeout is not None else self.timeout
         super().__init__(timeout)
 
-    def _create_session(self, Session_or_Options):
-        """创建内建Session对象
-        :param Session_or_Options: Session对象或SessionOptions对象
+    def _set_start_options(self, session_or_options, none):
+        """启动配置
+        :param session_or_options: Session、SessionOptions
+        :param none: 用于后代继承
         :return: None
         """
-        if Session_or_Options is None or isinstance(Session_or_Options, SessionOptions):
-            self._session_options = Session_or_Options or SessionOptions()
-            self._set_session(self._session_options)
+        if not session_or_options or isinstance(session_or_options, SessionOptions):
+            self._session_options = session_or_options or SessionOptions(session_or_options)
 
-        elif isinstance(Session_or_Options, Session):
-            self._session = Session_or_Options
-            self._session_options = SessionOptions(read_file=False)
+        elif isinstance(session_or_options, Session):
+            self._session_options = SessionOptions()
+            self._session = session_or_options
 
-        self._set_options()
-
-    def _set_session(self, opt):
-        """根据传入字典对session进行设置
-        :param opt: session配置字典
-        :return: None
-        """
-        self._session = Session()
-
-        if opt.headers:
-            self._session.headers = CaseInsensitiveDict(opt.headers)
-        if opt.cookies:
-            self.set_cookies(opt.cookies)
-        if opt.adapters:
-            for url, adapter in opt.adapters:
-                self._session.mount(url, adapter)
-
-        attrs = ['auth', 'proxies', 'hooks', 'params', 'verify',
-                 'cert', 'stream', 'trust_env', 'max_redirects']
-        for i in attrs:
-            attr = opt.__getattribute__(i)
-            if attr:
-                self._session.__setattr__(i, attr)
-
-    def _set_options(self):
-        """设置WebPage中与d模式共用的配置，便于WebPage覆盖掉"""
+    def _set_runtime_settings(self):
+        """设置运行时用到的属性"""
         self._timeout = self._session_options.timeout
         self._download_path = self._session_options.download_path
 
-    def set_cookies(self, cookies):
-        """为Session对象设置cookies
-        :param cookies: cookies信息
-        :return: None
-        """
-        set_session_cookies(self.session, cookies)
-
-    def set_headers(self, headers):
-        """设置通用的headers，设置的headers值回逐个覆盖原有的，不会清理原来的
-        :param headers: dict形式的headers
-        :return: None
-        """
-        headers = CaseInsensitiveDict(headers)
-        for i in headers:
-            self.session.headers[i] = headers[i]
-
-    def set_user_agent(self, ua):
-        """设置user agent"""
-        self.session.headers['user-agent'] = ua
+    def _create_session(self):
+        """创建内建Session对象"""
+        if not self._session:
+            self._session = self._session_options.make_session()
 
     def __call__(self, loc_or_str, timeout=None):
         """在内部查找元素
@@ -104,6 +70,12 @@ class SessionPage(BasePage):
         return self.ele(loc_or_str)
 
     # -----------------共有属性和方法-------------------
+    @property
+    def title(self):
+        """返回网页title"""
+        ele = self._ele('xpath://title', raise_err=False)
+        return ele.text if ele else None
+
     @property
     def url(self):
         """返回当前访问url"""
@@ -149,6 +121,13 @@ class SessionPage(BasePage):
         """返回访问url得到的response对象"""
         return self._response
 
+    @property
+    def set(self):
+        """返回用于等待的对象"""
+        if self._set is None:
+            self._set = SessionPageSetter(self)
+        return self._set
+
     def get(self, url, show_errmsg=False, retry=None, interval=None, timeout=None, **kwargs):
         """用get方式跳转到url
         :param url: 目标url
@@ -191,11 +170,12 @@ class SessionPage(BasePage):
         """
         return self._ele(loc_or_str, single=False)
 
-    def _ele(self, loc_or_ele, timeout=None, single=True):
+    def _find_elements(self, loc_or_ele, timeout=None, single=True, raise_err=None):
         """返回页面中符合条件的元素、属性或节点文本，默认返回第一个
         :param loc_or_ele: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
         :param timeout: 不起实际作用，用于和父类对应
         :param single: True则返回第一个，False则返回全部
+        :param raise_err: 找不到元素是是否抛出异常，为None时根据全局设置
         :return: SessionElement对象
         """
         return loc_or_ele if isinstance(loc_or_ele, SessionElement) else make_session_ele(self, loc_or_ele, single)
@@ -328,6 +308,142 @@ class SessionPage(BasePage):
                 raise ConnectionError(f'状态码：{r.status_code}')
             return r, f'状态码：{r.status_code}'
 
+    # --------------准备废弃----------------
+    def set_cookies(self, cookies):
+        """为Session对象设置cookies
+        :param cookies: cookies信息
+        :return: None
+        """
+        warn("set_cookies()方法即将弃用，请用set.cookies()方法代替。", DeprecationWarning)
+        self.set.cookies(cookies)
+
+    def set_headers(self, headers):
+        """设置通用的headers，设置的headers值回逐个覆盖原有的，不会清理原来的
+        :param headers: dict形式的headers
+        :return: None
+        """
+        warn("set_headers()方法即将弃用，请用set.headers()方法代替。", DeprecationWarning)
+        self.set.headers(headers)
+
+    def set_user_agent(self, ua):
+        """设置user agent"""
+        warn("set_user_agent()方法即将弃用，请用set.user_agent()方法代替。", DeprecationWarning)
+        self.set.user_agent(ua)
+
+
+class SessionPageSetter(object):
+    def __init__(self, page):
+        self._page = page
+
+    def timeout(self, second):
+        """设置连接超时时间
+        :param second: 秒数
+        :return: None
+        """
+        self._page.timeout = second
+
+    def cookies(self, cookies):
+        """为Session对象设置cookies
+        :param cookies: cookies信息
+        :return: None
+        """
+        set_session_cookies(self._page.session, cookies)
+
+    def headers(self, headers):
+        """设置通用的headers
+        :param headers: dict形式的headers
+        :return: None
+        """
+        self._page.session.headers = CaseInsensitiveDict(headers)
+
+    def header(self, attr, value):
+        """设置headers中一个项
+        :param attr: 设置名称
+        :param value: 设置值
+        :return: None
+        """
+        self._page.session.headers[attr.lower()] = value
+
+    def user_agent(self, ua):
+        """设置user agent
+        :param ua: user agent
+        :return: None
+        """
+        self._page.session.headers['user-agent'] = ua
+
+    def proxies(self, http, https=None):
+        """设置proxies参数
+        :param http: http代理地址
+        :param https: https代理地址
+        :return: None
+        """
+        proxies = None if http == https is None else {'http': http, 'https': https or http}
+        self._page.session.proxies = proxies
+
+    def auth(self, auth):
+        """设置认证元组或对象
+        :param auth: 认证元组或对象
+        :return: None
+        """
+        self._page.session.auth = auth
+
+    def hooks(self, hooks):
+        """设置回调方法
+        :param hooks: 回调方法
+        :return: None
+        """
+        self._page.session.hooks = hooks
+
+    def params(self, params):
+        """设置查询参数字典
+        :param params: 查询参数字典
+        :return: None
+        """
+        self._page.session.params = params
+
+    def verify(self, on_off):
+        """设置是否验证SSL证书
+        :param on_off: 是否验证 SSL 证书
+        :return: None
+        """
+        self._page.session.verify = on_off
+
+    def cert(self, cert):
+        """SSL客户端证书文件的路径(.pem格式)，或(‘cert’, ‘key’)元组
+        :param cert: 证书路径或元组
+        :return: None
+        """
+        self._page.session.cert = cert
+
+    def stream(self, on_off):
+        """设置是否使用流式响应内容
+        :param on_off: 是否使用流式响应内容
+        :return: None
+        """
+        self._page.session.stream = on_off
+
+    def trust_env(self, on_off):
+        """设置是否信任环境
+        :param on_off: 是否信任环境
+        :return: None
+        """
+        self._page.session.trust_env = on_off
+
+    def max_redirects(self, times):
+        """设置最大重定向次数
+        :param times: 最大重定向次数
+        :return: None
+        """
+        self._page.session.max_redirects = times
+
+    def add_adapter(self, url, adapter):
+        """添加适配器
+        :param url: 适配器对应url
+        :param adapter: 适配器对象
+        :return: None
+        """
+        self._page.session.mount(url, adapter)
+
 
 class DownloadSetter(object):
     """用于设置下载参数的类"""
